@@ -136,7 +136,7 @@ get_xkb_desc_rec (GsdA11yKeyboardManager *manager)
                 desc->ctrls = NULL;
                 status = XkbGetControls (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), XkbAllControlsMask, desc);
         }
-        gdk_error_trap_pop ();
+        gdk_error_trap_pop_ignored ();
 
         g_return_val_if_fail (desc != NULL, NULL);
         g_return_val_if_fail (desc->ctrls != NULL, NULL);
@@ -334,7 +334,7 @@ set_server_from_gsettings (GsdA11yKeyboardManager *manager)
         XkbFreeKeyboard (desc, XkbAllComponentsMask, True);
 
         XSync (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), FALSE);
-        gdk_error_trap_pop ();
+        gdk_error_trap_pop_ignored ();
 
         gnome_settings_profile_end (NULL);
 }
@@ -1007,7 +1007,7 @@ restore_server_xkb_config (GsdA11yKeyboardManager *manager)
                          XkbAllComponentsMask, True);
 
         XSync (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), FALSE);
-        gdk_error_trap_pop ();
+        gdk_error_trap_pop_ignored ();
 
         manager->priv->original_xkb_desc = NULL;
 }
