@@ -101,7 +101,7 @@ struct GsdXrandrManagerPrivate
 
         GSettings       *settings;
         GDBusNodeInfo   *introspection_data;
-        guint            owner_id;
+        GDBusConnection *connection;
 
         /* fn-F7 status */
         int             current_fn_f7_config;             /* -1 if no configs */
@@ -308,6 +308,35 @@ show_timestamps_dialog (GsdXrandrManager *manager, const char *msg)
 #endif
 }
 
+static void
+print_output (GnomeOutputInfo *info)
+{
+        g_print ("  Output: %s attached to %s\n", info->display_name, info->name);
+        g_print ("     status: %s\n", info->on ? "on" : "off");
+        g_print ("     width: %d\n", info->width);
+        g_print ("     height: %d\n", info->height);
+        g_print ("     rate: %d\n", info->rate);
+        g_print ("     primary: %s\n", info->primary ? "true" : "false");
+        g_print ("     position: %d %d\n", info->x, info->y);
+}
+
+static void
+print_configuration (GnomeRRConfig *config, const char *header)
+{
+        int i;
+
+        g_print ("=== %s Configuration ===\n", header);
+        if (!config) {
+                g_print ("  none\n");
+                return;
+        }
+
+        g_print ("  Clone: %s\n", config->clone ? "true" : "false");
+
+        for (i = 0; config->outputs[i] != NULL; ++i)
+                print_output (config->outputs[i]);
+}
+
 /* This function centralizes the use of gnome_rr_config_apply_from_filename_with_time().
  *
  * Optionally filters out GNOME_RR_ERROR_NO_MATCHING_CONFIG from
@@ -362,6 +391,10 @@ apply_configuration_and_display_error (GsdXrandrManager *manager, GnomeRRConfig
         GError *error;
         gboolean success;
 
+        gnome_rr_config_ensure_primary (config);
+
+        print_configuration (config, "Applying Configuration");
+
         error = NULL;
         success = gnome_rr_config_apply_with_time (config, priv->rw_screen, timestamp, &error);
         if (!success) {
@@ -606,7 +639,7 @@ gsd_xrandr_manager_2_apply_configuration (GsdXrandrManager *manager,
         gboolean result;
 
         if (parent_window_id != 0)
-                parent_window = gdk_window_foreign_new_for_display (gdk_display_get_default (), (GdkNativeWindow) parent_window_id);
+                parent_window = gdk_x11_window_foreign_new_for_display (gdk_display_get_default (), (GdkNativeWindow) parent_window_id);
         else
                 parent_window = NULL;
 
@@ -682,32 +715,6 @@ get_clone_size (GnomeRRScreen *screen, int *width, int *height)
         return FALSE;
 }
 
-static void
-print_output (GnomeOutputInfo *info)
-{
-        g_print ("  Output: %s attached to %s\n", info->display_name, info->name);
-        g_print ("     status: %s\n", info->on ? "on" : "off");
-        g_print ("     width: %d\n", info->width);
-        g_print ("     height: %d\n", info->height);
-        g_print ("     rate: %d\n", info->rate);
-        g_print ("     position: %d %d\n", info->x, info->y);
-}
-
-static void
-print_configuration (GnomeRRConfig *config, const char *header)
-{
-        int i;
-
-        g_print ("=== %s Configuration ===\n", header);
-        if (!config) {
-                g_print ("  none\n");
-                return;
-        }
-
-        for (i = 0; config->outputs[i] != NULL; ++i)
-                print_output (config->outputs[i]);
-}
-
 static gboolean
 config_is_all_off (GnomeRRConfig *config)
 {
@@ -1440,8 +1447,13 @@ auto_configure_outputs (GsdXrandrManager *manager, guint32 timestamp)
 
         /* Apply the configuration! */
 
-        if (applicable)
+        if (applicable) {
+                print_configuration (config, "auto configure");
+
                 apply_configuration_and_display_error (manager, config, timestamp);
+        } else {
+                g_debug ("Not an applicable config");
+        }
 
         g_list_free (just_turned_on);
         gnome_rr_config_free (config);
@@ -1483,6 +1495,8 @@ on_randr_event (GnomeRRScreen *screen, gpointer data)
                  config_timestamp);
 
         if (change_timestamp >= config_timestamp) {
+                GnomeRRConfig *rr_config;
+
                 /* The event is due to an explicit configuration change.
                  *
                  * If the change was performed by us, then we need to do nothing.
@@ -1490,6 +1504,18 @@ on_randr_event (GnomeRRScreen *screen, gpointer data)
                  * If the change was done by some other X client, we don't need
                  * to do anything, either; the screen is already configured.
                  */
+
+                /* Check if we need to update the primary */
+                rr_config = gnome_rr_config_new_current (priv->rw_screen);
+                if (gnome_rr_config_ensure_primary (rr_config)) {
+                        if (gnome_rr_config_applicable (rr_config, priv->rw_screen, NULL)) {
+                                print_configuration (rr_config, "Updating for primary");
+                                priv->last_config_timestamp = config_timestamp;
+                                gnome_rr_config_apply_with_time (rr_config, priv->rw_screen, config_timestamp, NULL);
+                        }
+                }
+                gnome_rr_config_free (rr_config);
+
                 show_timestamps_dialog (manager, "ignoring since change > config");
                 log_msg ("  Ignoring event since change >= config\n");
         } else {
@@ -1768,6 +1794,11 @@ gsd_xrandr_manager_stop (GsdXrandrManager *manager)
                 manager->priv->rw_screen = NULL;
         }
 
+        if (manager->priv->connection != NULL) {
+                g_object_unref (manager->priv->connection);
+                manager->priv->connection = NULL;
+        }
+
         log_open ();
         log_msg ("STOPPING XRANDR PLUGIN\n------------------------------------------------------------\n");
         log_close ();
@@ -1869,9 +1900,6 @@ gsd_xrandr_manager_finalize (GObject *object)
 
         g_return_if_fail (xrandr_manager->priv != NULL);
 
-        if (xrandr_manager->priv->owner_id > 0)
-                g_bus_unown_name (xrandr_manager->priv->owner_id);
-
         G_OBJECT_CLASS (gsd_xrandr_manager_parent_class)->finalize (object);
 }
 
@@ -1921,11 +1949,21 @@ static const GDBusInterfaceVTable interface_vtable =
 };
 
 static void
-on_bus_acquired (GDBusConnection     *connection,
-                 const gchar         *name,
-                 GsdXrandrManager    *manager)
+on_bus_gotten (GObject             *source_object,
+               GAsyncResult        *res,
+               GsdXrandrManager    *manager)
 {
+        GDBusConnection *connection;
         guint registration_id;
+        GError *error = NULL;
+
+        connection = g_bus_get_finish (res, &error);
+        if (connection == NULL) {
+                g_warning ("Could not get session bus: %s", error->message);
+                g_error_free (error);
+                return;
+        }
+        manager->priv->connection = connection;
 
         registration_id = g_dbus_connection_register_object (connection,
                                                              GSD_XRANDR_DBUS_PATH,
@@ -1942,14 +1980,10 @@ register_manager_dbus (GsdXrandrManager *manager)
         manager->priv->introspection_data = g_dbus_node_info_new_for_xml (introspection_xml, NULL);
         g_assert (manager->priv->introspection_data != NULL);
 
-        manager->priv->owner_id = g_bus_own_name (G_BUS_TYPE_SESSION,
-                                                  GSD_DBUS_NAME,
-                                                  G_BUS_NAME_OWNER_FLAGS_NONE,
-                                                  (GBusAcquiredCallback) on_bus_acquired,
-                                                  NULL,
-                                                  NULL,
-                                                  manager,
-                                                  NULL);
+        g_bus_get (G_BUS_TYPE_SESSION,
+                   NULL,
+                   (GAsyncReadyCallback) on_bus_gotten,
+                   manager);
 }
 
 GsdXrandrManager *
